-- ************************************************************
-- vendor.sql -	R A D A R  V E N D O R  S C H E M A - 
-- ************************************************************
-- This schema provides the structure for the RADAR vendor system.
-- Developed by: Kyle Cunningham <kcunning@cs>
-- Date: 01/26/2006 (for Postgres)
-- ***********************************************************
--			 		M O D I F I C A T I O N S 
-- ***********************************************************
--   DATE     | AUTHOR |             REASON
-- -----------------------------------------------------------
-- 03/27/2006 |kcunning| Added user_id attribute to vendor.order.
--	      	  |	       | Also changed totalprice to total1, 2, 3
--	          |	       | b/c orders are quoted by 3 vendors
-- -----------------------------------------------------------
-- 04/3/2006  |kcunning| Added "type" attribute to vendor.order
--  	      |	       | to provide ability to sort orders by type	
-- -----------------------------------------------------------
-- 04/5/2006  |kcunning| Removed OrderDetails relation, moved
--            |        | attributes in to Order relation
-- -----------------------------------------------------------
-- 08/13/2006 |kcunning| Removed total1, 2, 3 from order, new
--			  |        | vendor_order_total relation created to allow	
--            |        | *..* relationship
-- -----------------------------------------------------------
-- 09/13/2006 |kcunning| Added 'humanreadable' description to 
--			  |        | vendor.item relation				
-- -----------------------------------------------------------
-- 10/03/2006 |kcunning| Added 'type' identifier to 
--			  |        | vendor.item relation	
-- -----------------------------------------------------------

DROP SCHEMA vendor CASCADE;
CREATE SCHEMA vendor;

-- ************************************************************
-- User Relation
-- ************************************************************
-- This relation contains information for each user that places
-- and order through the vendor system.
-- ------------------------------------------------------------

CREATE TABLE vendor.user (
	id		SERIAL PRIMARY KEY,	-- unique identifier
	firstname	TEXT,
	lastname	TEXT,
	email		TEXT
);


-- ************************************************************
-- Order Relation
-- ************************************************************
-- This relation contains information for all orders placed 
-- through the vendor system.
-- ------------------------------------------------------------

CREATE TABLE vendor.order (
	id		SERIAL PRIMARY KEY,	-- unique identifier
	user_id		INTEGER,	-- FK vendor.user
	type		TEXT,		-- {security, audioVisual, catering, floral, rental}
	title		TEXT,	-- title of event
	location_id	TEXT,	-- FK radar.location(id)
	startdate	DATE,	-- 
	enddate		DATE,
	setupstart	TEXT,
	setupcomplete	TEXT,
	-- ------------------------------------------------------------------------
	-- kcunning 08/13/2006 - modified relation for more than 3 vendor totals
	--                     - new vendor_order_total relation created
	-- ------------------------------------------------------------------------
	-- total1		INTEGER,	-- 3 vendors quote each order, so each order has
	-- total2		INTEGER,	-- 3 different totals, one for each vendor, except
	-- total3		INTEGER,	-- for security, it is a CMU "monopoly"
	-- ------------------------------------------------------------------------
	quoteaccepted	TEXT	-- id of vendor whose quote was accepted
);

-- ************************************************************
-- Vendor_Order_Total Relation
-- ************************************************************
-- This relation links vendor_id and order_id, providing a specific
-- vendor's price for a specific order.
-- ------------------------------------------------------------

CREATE TABLE vendor.vendor_order_total (
	id		SERIAL PRIMARY KEY,	-- unique identifier
	order_id	INTEGER,		-- FK vendor.order(id)
	vendor_typeid	TEXT,		-- FK vendor.vendorinfo(typeid)
	total		INTEGER				
);


-- ************************************************************
-- OrderDetails Relation
-- ************************************************************
-- This relation contains order details for all orders placed 
-- through the vendor system.
-- ------------------------------------------------------------
-- KAC 04/05/2006 - commented out OrderDetails Relation, moved
--                  attributes to Order due to 1..1 relationship.

--CREATE TABLE vendor.orderdetails (
--	id		SERIAL PRIMARY KEY,	-- unique identifier
--	order_id	INTEGER,	-- FK order(id)
--	title		TEXT,	-- title of event
--	location_id	TEXT,	-- FK radar.location(id)
--	startdate	DATE,	-- 
--	enddate		DATE,
--	setupstart	TEXT,
--	setupcomplete	TEXT
--);


-- ************************************************************
-- Line Relation
-- ************************************************************
-- This relation contains individual lines, which contain items 
-- requested, for each order placed through the vendor system.
-- ------------------------------------------------------------

CREATE TABLE vendor.line (
	id		SERIAL PRIMARY KEY,	-- unique identifier
	order_id	INTEGER,	-- FK order(id)
	item_id		INTEGER,	-- FK item(id)
	quantity	INTEGER
);


-- ************************************************************
-- Item Relation
-- ************************************************************
-- This relation contains details for all items that can be ordered 
-- through the vendor system.  Each item will be unique, so if there
-- are 3 different sizes for a tent and 3 different colors there will be 
-- 9 tent entries.  Each tuple will also have the separate vendor
-- costs for that item.
-- ------------------------------------------------------------

CREATE TABLE vendor.item (
	id		SERIAL PRIMARY KEY,	-- unique identifier
	name		TEXT,	-- itemName
		-- ---------------------------------------------------------
		-- kcunning 09-14-2006 - added humanreadable text attribute
		-- ---------------------------------------------------------
	humanreadable	TEXT,	-- human-readable form of item name
		-- ---------------------------------------------------------
		-- kcunning 10-03-2006 - added type text attribute
		-- ---------------------------------------------------------
	type		TEXT	-- type of vendor that offers this item,
						-- i.e. security, floral, catering
);


-- ************************************************************
-- VendorInfo Relation
-- ************************************************************
-- This relation contains information for each vendor offered 
-- through the vendor system.
-- ------------------------------------------------------------

CREATE TABLE vendor.vendorinfo (
	id		SERIAL PRIMARY KEY,	-- unique identifier
	name		TEXT,
	email		TEXT,
	typeid		TEXT,	-- av1,2,3, catering1,2,3, ...
	stockmessagebody	TEXT,
	responsivity	TEXT
);


-- ************************************************************
-- MessageInjection Relation
-- ************************************************************
-- This relation contains messages produced each time a vendor 	
-- order is submitted and/or accepted.  These messages are sent
-- to the user at various points throughout the session (based
-- on minutes and seconds)
-- ------------------------------------------------------------

CREATE TABLE vendor.messageinjection (
  	id		SERIAL PRIMARY KEY,
  	sender 		TEXT NOT NULL,
  	senderemail 	TEXT NOT NULL,
  	message 	TEXT NOT NULL,
  	minutes 	INTEGER NOT NULL,
  	seconds 	INTEGER NOT NULL,
  	contactemail 	TEXT NOT NULL
);


-- ************************************************************
-- ItemPrice Relation
-- ************************************************************
-- This relation holds data for each vendor's price for a specific
-- item.  item_id references vendor.item(id) and vendorid 
-- references vendor.vendorinfo(typeid) to match typeid.
-- ------------------------------------------------------------

CREATE TABLE vendor.itemprice (
  	id	 	SERIAL PRIMARY KEY,
  	item_id	 	INTEGER,	-- FK item(id)
  	vendor_type_id 	TEXT,	-- FK vendorinformation(typeid)
  	price		INTEGER
);

-- ************************************************************
-- 			END Vendor.sql
-- ************************************************************
