package airspaceGUI.base.corpus.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.text.DateFormatSymbols;
import java.util.Arrays;
import java.util.StringTokenizer;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerListModel;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import airspaceGUI.base.corpus.MessageType;
import airspaceGUI.expansion.ConstraintFactory;
import airspaceGUI.expansion.TimePanel;


/**
 * Displays the time of a message in a semi-human readable format. Also, this
 * controls whether a message is considered "backstory" or not
 * 
 * @author srbennet
 * 
 */
public class BackstoryTimePanel extends JPanel
{
    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    // set if this is a backstory message
    private final transient JCheckBox backstoryCheck = new JCheckBox("Backstory");

    // set up all the spinner models
    private final transient SpinnerNumberModel sbModel = new SpinnerNumberModel(
            1, 1, 90, 1);

    private final transient String[] months = new DateFormatSymbols()
            .getShortMonths();

    private final transient SpinnerModel Month = new SpinnerListModel(Arrays
            .asList(months));

    private final transient SpinnerNumberModel days = new SpinnerNumberModel(1,
            1, 31, 1);

    // initalize the actual spinners
    private final transient JSpinner month = new JSpinner(Month);

    private final transient JSpinner day = new JSpinner(days);

    private final transient JSpinner sinceBeginning = new JSpinner(sbModel);

    private transient boolean isBackstory = false;

    private transient final ChangeListener backstoryListener = new ChangeListener() {
        public void stateChanged(final ChangeEvent arg0) {

            isBackstory = backstoryCheck.isSelected();
            selectPanel();
        }
    };

    private final transient TimePanel timePanel = new TimePanel();

    /**
     * This is the panel for the time fields of a corpus fake message
     * 
     * @param arg0
     *            true if this panel is a backstoryPanel, false otherwise
     * 
     */
    public BackstoryTimePanel(boolean arg0)
    {
        super();
        isBackstory = arg0;
        // if the backstory checkbox is checked, change the time panel
        // appropriately
        backstoryCheck.addChangeListener(backstoryListener);
        this.setLayout(new GridBagLayout());
        selectPanel();
    }

    /**
     * Lays out the BackstoryPanel correctly
     *
     */
    private void setUpBackstoryPanel() {
        GridBagConstraints gbc = new GridBagConstraints();
        int col = 0;
        final ConstraintFactory f = new ConstraintFactory(8, 8, 8, 8, 2);
        this.setIgnoreRepaint(true);
        this.removeAll();

        gbc = f.getConstraint(col++, 0, ConstraintFactory.FIELD,
                ConstraintFactory.MIDDLE);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0, 0, 0, 0);
        gbc.anchor = GridBagConstraints.NORTHWEST;
        this.add(backstoryCheck, gbc);

        gbc = f.getConstraint(col++, 0, 2, ConstraintFactory.FIELD,
                ConstraintFactory.MIDDLE);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0, 0, 0, 0);
        gbc.anchor = GridBagConstraints.NORTHWEST;
        this.add(timePanel, gbc);

        this.setIgnoreRepaint(false);
        this.revalidate();
        this.repaint();
    }

    /**
     * Handles the setup of a normal time panel
     * 
     */
    private void setUpNormalPanel() {
        GridBagConstraints gbc = new GridBagConstraints();
        ConstraintFactory f = new ConstraintFactory(8, 8, 8, 8, 2);
        int col = 0;
        JLabel minutesLabel = new JLabel(" Minutes: ");
        this.setIgnoreRepaint(true);
        this.removeAll();
        // normalTimePanel.setLayout(new GridBagLayout());

        gbc = f.getConstraint(col++, 0, ConstraintFactory.FIELD,
                ConstraintFactory.MIDDLE);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.insets = new Insets(0, 0, 0, 0);
        this.add(backstoryCheck, gbc);

        gbc = f.getConstraint(col++, 0, ConstraintFactory.FIELD,
                ConstraintFactory.MIDDLE);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        this.add(month, gbc);
        gbc = f.getConstraint(col++, 0, ConstraintFactory.FIELD,
                ConstraintFactory.MIDDLE);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        this.add(day, gbc);

        col++;
        gbc = f.getConstraint(col++, 0, ConstraintFactory.FIELD,
                ConstraintFactory.MIDDLE);
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.NORTHEAST;
        // gbc.insets = new Insets(8, 0, 8, 0);
        this.add(minutesLabel, gbc);
        gbc = f.getConstraint(col++, 0, ConstraintFactory.FIELD,
                ConstraintFactory.MIDDLE);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        // gbc.insets = new Insets(8, 2, 8, 4);
        this.add(sinceBeginning, gbc);

        this.setIgnoreRepaint(false);
        this.revalidate();
        this.repaint();
    }

    /**
     * A function to ensure the correct Panel is showing.
     * 
     * NOTE: for this function to act correctly, isBackstory must be set
     * correctly.
     */
    private void selectPanel() {
        if (backstoryCheck.isSelected() || isBackstory)
        {
            setUpBackstoryPanel();
        } else
        {
            setUpNormalPanel();
        }
    }

    /**
     * This correctly populates the fields on the time panel with the time that
     * the paremeter was sent atLabel.
     * 
     * @param message
     *            The message to get the time from.
     */
    public void setTime(MessageType message) {
        // Code to correctly fill in the time fields
        final StringTokenizer time = new StringTokenizer(message.getTime(),
                ": ");
        month.setValue(time.nextToken());
        day.setValue(new Integer(time.nextToken()));
        backstoryCheck.setSelected(message.getBackstory());
        isBackstory = message.getBackstory();
        selectPanel();

        if (message.getBackstory()) // fill in the correct time fields
        // according to type
        {
            timePanel.setText(message.getTime(), TimePanel.MESSAGETIME);
        } else
        {
            final Integer minSinceBeginning = new Integer(time.nextToken());
            sinceBeginning.setValue(minSinceBeginning);
        }
    }

    /**
     * Returns the String representation of what is displayed on this panel.
     * 
     * @return a String representation of the time.
     */
    public String getTime() {

        if (backstoryCheck.isSelected())
        {
            return timePanel.getTime(TimePanel.MESSAGETIME);
        } else
        {
            return (month.getValue() + " " + day.getValue() + " " + sinceBeginning
                    .getValue());
        }
    }

    /**
     * Clears the time to default values
     * 
     */
    public void clearTime() {
        timePanel.clearTime();
        month.setValue("Jun");
        day.setValue(new Integer(7));
        backstoryCheck.setSelected(false);
    }

    /**
     * Adds a changeListener to all the components of this Panel.
     * 
     * @param listener
     *            The changeListener to be notified when something changes
     */
    public void addChangeListener(ChangeListener listener) {
        timePanel.addChangeListener(listener);
        month.addChangeListener(listener);
        day.addChangeListener(listener);
        sinceBeginning.addChangeListener(listener);
    }
}
