package airspaceGUI.expansion;

import java.awt.GridBagConstraints;
import java.awt.Insets;

/**
 * A class to generate the correct constraints to place a component correctly in
 * the interface
 * 
 * @author srbennet
 * 
 */
public class ConstraintFactory
{

    private final transient int topInset;

    private final transient int leftInset;

    private final transient int bottomInset;

    private final transient int rightInset;

    private final transient int innerInset;

    /**
     * defines where a component is in the layout
     */
    public static final int MIDDLE = 0;

    /**
     * defines where a component is in the layout
     */
    public static final int TOPMOST = 1;

    /**
     * defines where a component is in the layout
     */
    public static final int BOTTOMMOST = 2;

    /**
     * defines what kind of role the component is playing
     */
    public static final int LABEL = 3;

    /**
     * defines what kind of role the component is playing
     */
    public static final int FIELD = 4;

    /**
     * Initializes the insets for this ConstraintFactory
     * 
     * @param topInset
     * @param leftInset
     * @param bottomInset
     * @param rightInset
     * @param innerInset
     */
    public ConstraintFactory(int topInset, int leftInset, int bottomInset,
            int rightInset, int innerInset)
    {
        this.topInset = topInset;
        this.leftInset = leftInset;
        this.bottomInset = bottomInset;
        this.rightInset = rightInset;
        this.innerInset = innerInset;
    }

    /**
     * 
     * @param x -
     *            x-coordinate of this constraint
     * @param y -
     *            y-coordinate of this constraint
     * @param role -
     *            the constant of the role this constraint will play in the
     *            interface
     * @param position -
     *            the position of the object on the line
     * @return constraints correctly configured for this object
     */
    public GridBagConstraints getConstraint(final int x, final int y,
            final int role, final int position) {
        return getConstraint(x, y, 1, 0.0, role, position);
    }

    /**
     * 
     * @param x -
     *            x-coordinate of this constraint
     * @param y -
     *            y-coordinate of this constraint
     * @param span -
     *            the number of columns this object should span
     * @param role -
     *            the constant of the role this constraint will play in the
     *            interface
     * @param position -
     *            the position of the object on the line
     * @return constraints correctly configured for this object
     */
    public GridBagConstraints getConstraint(final int x, final int y,
            final int span, final int role, final int position) {
        return getConstraint(x, y, span, 0.0, role, position);
    }

    /**
     * 
     * @param x -
     *            x-coordinate of this constraint
     * @param y -
     *            y-coordinate of this constraint
     * @param span -
     *            the number of columns this object should span
     * @param vweight -
     *            how apt this object is to take up more space when the window
     *            is resized
     * @param role -
     *            the constant of the role this constraint will play in the
     *            interface
     * @param position -
     *            the position of the object on the line
     * @return constraints correctly configured for this object
     */
    public GridBagConstraints getConstraint(final int x, final int y,
            final int span, final double vweight, final int role,
            final int position) {
        final GridBagConstraints ret = new GridBagConstraints();
        ret.gridx = x;
        ret.gridy = y;
        ret.gridwidth = span;
        ret.weighty = vweight;
        final Insets insets = new Insets(innerInset, innerInset, innerInset,
                innerInset);
        if (role == LABEL)
        {
            insets.left = leftInset;
            ret.anchor = GridBagConstraints.EAST;
            ret.fill = GridBagConstraints.NONE;
            ret.weightx = 0.0;
        }
        if (role == FIELD)
        {
            insets.right = rightInset;
            ret.anchor = GridBagConstraints.WEST;
            ret.fill = GridBagConstraints.HORIZONTAL;
            ret.weightx = 1.0;
        }
        if (vweight > 0.0)
        { // this is a special field or the label thereto, so...
            if (role == LABEL)
            {
                ret.anchor = GridBagConstraints.NORTHEAST;
            }
            if (role == FIELD)
            {
                ret.anchor = GridBagConstraints.NORTHWEST;
                ret.fill = GridBagConstraints.BOTH;
            }
        }
        if (position == TOPMOST)
        {
            insets.top = topInset;
        }
        if (position == BOTTOMMOST)
        {
            insets.bottom = bottomInset;
        }
        ret.insets = insets;

        return ret;
    }
}